@@ -1,12 +1,17 @@
 /*
  *    Example program for the Allegro library, by Shawn Hargreaves.
  *
- *    This program demonstrates the use of triple buffering and vertical
- *    retrace interrupt simulation.
+ *    This program demonstrates the use of triple buffering. Several
+ *    triangles are displayed rotating and bouncing on the screen
+ *    until you press a key. Note that on some platforms you
+ *    can't get real hardware triple buffering.  The Allegro code
+ *    remains the same, but most likely the graphic driver will
+ *    emulate it. Unfortunately, in these cases you can't expect
+ *    the animation to be completely smooth and flicker free.
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -24,6 +29,7 @@ typedef struct SHAPE
 
 
 SHAPE shapes[NUM_SHAPES];
+int triplebuffer_not_available = 0;
 
 
 
@@ -80,6 +86,7 @@ void move_shape(SHAPE *shape)
 void draw(BITMAP *b)
 {
    int c;
+   char message[1024];
 
    acquire_bitmap(b);
 
@@ -87,48 +94,35 @@ void draw(BITMAP *b)
 
    for (c=0; c<NUM_SHAPES; c++) {
       triangle(b, 
-	       fixtoi(shapes[c].x+fixmul(shapes[c].dist1, fixcos(shapes[c].dir1))),
-	       fixtoi(shapes[c].y+fixmul(shapes[c].dist1, fixsin(shapes[c].dir1))),
-	       fixtoi(shapes[c].x+fixmul(shapes[c].dist2, fixcos(shapes[c].dir2))),
-	       fixtoi(shapes[c].y+fixmul(shapes[c].dist2, fixsin(shapes[c].dir2))),
-	       fixtoi(shapes[c].x+fixmul(shapes[c].dist3, fixcos(shapes[c].dir3))),
-	       fixtoi(shapes[c].y+fixmul(shapes[c].dist3, fixsin(shapes[c].dir3))),
+	       fixtoi(shapes[c].x+fixmul(shapes[c].dist1,
+					 fixcos(shapes[c].dir1))),
+	       fixtoi(shapes[c].y+fixmul(shapes[c].dist1,
+					 fixsin(shapes[c].dir1))),
+	       fixtoi(shapes[c].x+fixmul(shapes[c].dist2,
+					 fixcos(shapes[c].dir2))),
+	       fixtoi(shapes[c].y+fixmul(shapes[c].dist2,
+					 fixsin(shapes[c].dir2))),
+	       fixtoi(shapes[c].x+fixmul(shapes[c].dist3,
+					 fixcos(shapes[c].dir3))),
+	       fixtoi(shapes[c].y+fixmul(shapes[c].dist3,
+					 fixsin(shapes[c].dir3))),
 	       shapes[c].color);
 
       move_shape(shapes+c);
    } 
 
-   if (retrace_proc)
-      textout_ex(b, font, "Triple buffering with retrace sync", 0, 0, 255, -1);
+   if (triplebuffer_not_available)
+      ustrzcpy (message, sizeof message, "Simulated triple buffering");
    else
-      textout_ex(b, font, "Triple buffering", 0, 0, 255, -1);
+      ustrzcpy (message, sizeof message, "Real triple buffering");
+
+   textout_ex(b, font, message, 0, 0, 255, -1);
 
    release_bitmap(b);
 }
 
 
 
-int fade_color = 63;
-
-/* vertical retrace callback function for doing the palette fade */
-void fade(void)
-{
-   int c = (fade_color < 64) ? fade_color : 127 - fade_color;
-   RGB rgb;
-
-   rgb.r = rgb.g = rgb.b = c;
-
-   _set_color(0, &rgb);
-
-   fade_color++;
-   if (fade_color >= 128)
-      fade_color = 0;
-}
-
-END_OF_FUNCTION(fade);
-
-
-
 /* main animation control loop */
 void triple_buffer(BITMAP *page1, BITMAP *page2, BITMAP *page3)
 {
@@ -164,6 +158,15 @@ int main(void)
 {
    BITMAP *page1, *page2, *page3;
    int c;
+   int w, h;
+
+#ifdef ALLEGRO_DOS
+   w = 320;
+   h = 240;
+#else
+   w = 640;
+   h = 480;
+#endif
 
    if (allegro_init() != 0)
       return 1;
@@ -172,13 +175,16 @@ int main(void)
    install_mouse();
 
    /* see comments in exflip.c */
-   if (set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 720) != 0) {
-      if (set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 0) != 0) {
-	 if (set_gfx_mode(GFX_SAFE, 320, 240, 0, 0) != 0) {
-	    set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-	    allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
-	    return 1;
-         }
+#ifdef ALLEGRO_VRAM_SINGLE_SURFACE
+   if (set_gfx_mode(GFX_AUTODETECT, w, h, 0, h * 3) != 0) {
+#else
+   if (set_gfx_mode(GFX_AUTODETECT, w, h, 0, 0) != 0) {
+#endif
+      if (set_gfx_mode(GFX_SAFE, w, h, 0, 0) != 0) {
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Unable to set any graphic mode\n%s\n",
+			 allegro_error);
+	 return 1;
       }
    }
 
@@ -190,15 +196,7 @@ int main(void)
 
    /* if that didn't work, give up */
    if (!(gfx_capabilities & GFX_CAN_TRIPLE_BUFFER)) {
-      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-
-      #ifdef ALLEGRO_DOS
-	 allegro_message("Triple buffering not available, sorry\n\nTry again in clean DOS mode (not under Windows)\n");
-      #else
-	 allegro_message("Triple buffering not available, sorry\n");
-      #endif
-
-      return 1;
+      triplebuffer_not_available = TRUE;
    }
 
    /* allocate three sub bitmaps to access pages of the screen */
@@ -216,29 +214,13 @@ int main(void)
    for (c=0; c<NUM_SHAPES; c++)
       init_shape(shapes+c);
 
-   LOCK_VARIABLE(fade_color);
-   LOCK_FUNCTION(fade);
-
-   if (timer_can_simulate_retrace()) {
-      timer_simulate_retrace(TRUE);
-      retrace_proc = fade;
-   }
-   else {
-      alert("Retrace sync is not available in",
-	    "this environment, so you won't get",
-	    "the funky smooth palette fading", 
-	    "Damn, that's a shame", NULL, 13, 0);
-   }
-
    triple_buffer(page1, page2, page3);
 
    destroy_bitmap(page1);
    destroy_bitmap(page2);
    destroy_bitmap(page3);
 
-   retrace_proc = NULL;
-
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
