@@ -59,6 +59,10 @@
    #else
       #error platform not supported
    #endif
+
+   #ifndef SCAN_DEPEND
+      #include "allegro/platform/astdint.h"
+   #endif
 #endif
 
 
@@ -68,9 +72,13 @@
 
    #ifndef AL_INLINE
       #ifdef __cplusplus
-         #define AL_INLINE(type, name, args, code)    static inline type name args code
+         #define AL_INLINE(type, name, args, code)    \
+            static inline type name args;             \
+            static inline type name args code
       #else
-         #define AL_INLINE(type, name, args, code)    extern __inline__ type name args code
+         #define AL_INLINE(type, name, args, code)    \
+            extern __inline__ type name args;         \
+            extern __inline__ type name args code
       #endif
    #endif
 
@@ -105,6 +113,12 @@
 
    #ifdef __i386__
       #define ALLEGRO_I386
+      #define _AL_SINCOS(x, s, c)  __asm__ ("fsincos" : "=t" (c), "=u" (s) : "0" (x))
+   #endif
+
+   #ifdef __amd64__
+      #define ALLEGRO_AMD64
+      #define _AL_SINCOS(x, s, c)  __asm__ ("fsincos" : "=t" (c), "=u" (s) : "0" (x))
    #endif
 
    #ifndef AL_CONST
@@ -118,6 +132,22 @@
          #define AL_INLINE_DEPRECATED(type, name, args, code)      AL_INLINE(__attribute__ ((deprecated)) type, name, args, code)
       #endif
    #endif
+
+   #ifndef AL_ALIAS
+      #define AL_ALIAS(DECL, CALL)                      \
+      static __attribute__((unused)) __inline__ DECL    \
+      {                                                 \
+         return CALL;                                   \
+      }
+   #endif
+
+   #ifndef AL_ALIAS_VOID_RET
+      #define AL_ALIAS_VOID_RET(DECL, CALL)                  \
+      static __attribute__((unused)) __inline__ void DECL    \
+      {                                                      \
+         CALL;                                               \
+      }
+   #endif
 #endif
 
 
@@ -133,6 +163,10 @@
  * only be included if none of the above headers defined custom versions.
  */
 
+#ifndef _AL_SINCOS
+   #define _AL_SINCOS(x, s, c)  do { (c) = cos(x); (s) = sin(x); } while (0)
+#endif
+
 #ifndef INLINE
    #define INLINE
 #endif
@@ -187,6 +221,22 @@
    #define AL_INLINE_DEPRECATED(type, name, args, code)      AL_INLINE(type, name, args, code)
 #endif
 
+#ifndef AL_ALIAS
+   #define AL_ALIAS(DECL, CALL)              \
+   static INLINE DECL                        \
+   {                                         \
+      return CALL;                           \
+   }
+#endif
+
+#ifndef AL_ALIAS_VOID_RET
+   #define AL_ALIAS_VOID_RET(DECL, CALL)     \
+   static INLINE void DECL                   \
+   {                                         \
+      CALL;                                  \
+   }
+#endif
+
 #ifndef END_OF_MAIN
    #define END_OF_MAIN()
 #endif
@@ -227,6 +277,8 @@
    #define FA_DIREC        16
    #define FA_ARCH         32
 #endif
+   #define FA_NONE         0
+   #define FA_ALL          (~FA_NONE)
 
 
 #ifdef __cplusplus
@@ -264,12 +316,12 @@
 /* not many places actually use these, but still worth emulating */
 #ifndef ALLEGRO_DJGPP
    #define _farsetsel(seg)
-   #define _farnspokeb(addr, val)   (*((unsigned char  *)(addr)) = (val))
-   #define _farnspokew(addr, val)   (*((unsigned short *)(addr)) = (val))
-   #define _farnspokel(addr, val)   (*((unsigned long  *)(addr)) = (val))
-   #define _farnspeekb(addr)        (*((unsigned char  *)(addr)))
-   #define _farnspeekw(addr)        (*((unsigned short *)(addr)))
-   #define _farnspeekl(addr)        (*((unsigned long  *)(addr)))
+   #define _farnspokeb(addr, val)   (*((uint8_t  *)(addr)) = (val))
+   #define _farnspokew(addr, val)   (*((uint16_t *)(addr)) = (val))
+   #define _farnspokel(addr, val)   (*((uint32_t *)(addr)) = (val))
+   #define _farnspeekb(addr)        (*((uint8_t  *)(addr)))
+   #define _farnspeekw(addr)        (*((uint16_t *)(addr)))
+   #define _farnspeekl(addr)        (*((uint32_t *)(addr)))
 #endif
 
 
@@ -305,17 +357,17 @@
 #endif
 
 #ifndef bmp_write8
-   #define bmp_write8(addr, c)         (*((unsigned char  *)(addr)) = (c))
-   #define bmp_write15(addr, c)        (*((unsigned short *)(addr)) = (c))
-   #define bmp_write16(addr, c)        (*((unsigned short *)(addr)) = (c))
-   #define bmp_write32(addr, c)        (*((unsigned long  *)(addr)) = (c))
+   #define bmp_write8(addr, c)         (*((uint8_t  *)(addr)) = (c))
+   #define bmp_write15(addr, c)        (*((uint16_t *)(addr)) = (c))
+   #define bmp_write16(addr, c)        (*((uint16_t *)(addr)) = (c))
+   #define bmp_write32(addr, c)        (*((uint32_t *)(addr)) = (c))
 
-   #define bmp_read8(addr)             (*((unsigned char  *)(addr)))
-   #define bmp_read15(addr)            (*((unsigned short *)(addr)))
-   #define bmp_read16(addr)            (*((unsigned short *)(addr)))
-   #define bmp_read32(addr)            (*((unsigned long  *)(addr)))
+   #define bmp_read8(addr)             (*((uint8_t  *)(addr)))
+   #define bmp_read15(addr)            (*((uint16_t *)(addr)))
+   #define bmp_read16(addr)            (*((uint16_t *)(addr)))
+   #define bmp_read32(addr)            (*((uint32_t *)(addr)))
 
-   AL_INLINE(int, bmp_read24, (unsigned long addr),
+   AL_INLINE(int, bmp_read24, (uintptr_t addr),
    {
       unsigned char *p = (unsigned char *)addr;
       int c;
@@ -325,7 +377,7 @@
       return c;
    })
 
-   AL_INLINE(void, bmp_write24, (unsigned long addr, int c),
+   AL_INLINE(void, bmp_write24, (uintptr_t addr, int c),
    {
       unsigned char *p = (unsigned char *)addr;
 
