@@ -8,7 +8,9 @@
  *                                           /\____/
  *                                           \_/__/
  *
- *      Low-level functions for the AWE32 driver, by George Foot.
+ *      Low-level functions for the AWE32 driver.
+ *
+ *      By George Foot.
  *
  *      Information taken primarily from "AWE32/EMU8000 Programmer's Guide"
  *      (AEPG) by Dave Rossum. The AEPG is part of the AWE32 Developers'
@@ -555,7 +557,7 @@ static INLINE int read_CCCA(int channel)
 
 
 /*
-static INLINE int read_HWCF4()
+static INLINE int read_HWCF4(void)
 {
    return read_dword(1, 9, 1);
 }
@@ -564,7 +566,7 @@ static INLINE int read_HWCF4()
 
 
 /*
-static INLINE int read_HWCF5()
+static INLINE int read_HWCF5(void)
 {
    return read_dword(1, 10, 1);
 }
@@ -573,7 +575,7 @@ static INLINE int read_HWCF5()
 
 
 /*
-static INLINE int read_HWCF6()
+static INLINE int read_HWCF6(void)
 {
    return read_dword(1, 13, 1);
 }
@@ -582,7 +584,7 @@ static INLINE int read_HWCF6()
 
 
 /*
-static INLINE int read_SMALR()
+static INLINE int read_SMALR(void)
 {
    return read_dword(1, 20, 1);
 }
@@ -591,7 +593,7 @@ static INLINE int read_SMALR()
 
 
 /*
-static INLINE int read_SMARR()
+static INLINE int read_SMARR(void)
 {
    return read_dword(1, 21, 1);
 }
@@ -600,7 +602,7 @@ static INLINE int read_SMARR()
 
 
 /*
-static INLINE int read_SMALW()
+static INLINE int read_SMALW(void)
 {
    return read_dword(1, 22, 1);
 }
@@ -609,7 +611,7 @@ static INLINE int read_SMALW()
 
 
 /*
-static INLINE int read_SMARW()
+static INLINE int read_SMARW(void)
 {
    return read_dword(1, 23, 1);
 }
@@ -618,7 +620,7 @@ static INLINE int read_SMARW()
 
 
 /*
-static INLINE int read_SMLD()
+static INLINE int read_SMLD(void)
 {
    return read_word(1, 26, 1);
 }
@@ -627,7 +629,7 @@ static INLINE int read_SMLD()
 
 
 /*
-static INLINE int read_SMRD()
+static INLINE int read_SMRD(void)
 {
    return read_word(1, 26, 2);
 }
@@ -636,7 +638,7 @@ static INLINE int read_SMRD()
 
 
 /*
-static INLINE int read_WC()
+static INLINE int read_WC(void)
 {
    return read_word(1, 27, 2);
 }
@@ -645,7 +647,7 @@ static INLINE int read_WC()
 
 
 /*
-static INLINE int read_HWCF1()
+static INLINE int read_HWCF1(void)
 {
    return read_word(1, 29, 1);
 }
@@ -654,7 +656,7 @@ static INLINE int read_HWCF1()
 
 
 /*
-static INLINE int read_HWCF2()
+static INLINE int read_HWCF2(void)
 {
    return read_word(1, 30, 1);
 }
@@ -663,7 +665,7 @@ static INLINE int read_HWCF2()
 
 
 /*
-static INLINE int read_HWCF3()
+static INLINE int read_HWCF3(void)
 {
    return read_word(1, 31, 1);
 }
@@ -672,7 +674,7 @@ static INLINE int read_HWCF3()
 
 
 /*
-static INLINE int read_INIT1()
+static INLINE int read_INIT1(void)
 {
    return read_word(2, 0, 1);
 }
@@ -681,7 +683,7 @@ static INLINE int read_INIT1()
 
 
 /*
-static INLINE int read_INIT2()
+static INLINE int read_INIT2(void)
 {
    return read_word(2, 0, 2);
 }
@@ -690,7 +692,7 @@ static INLINE int read_INIT2()
 
 
 /*
-static INLINE int read_INIT3()
+static INLINE int read_INIT3(void)
 {
    return read_word(3, 0, 1);
 }
@@ -699,7 +701,7 @@ static INLINE int read_INIT3()
 
 
 /*
-static INLINE int read_INIT4()
+static INLINE int read_INIT4(void)
 {
    return read_word(3, 0, 2);
 }
@@ -835,7 +837,7 @@ static INLINE int read_FM2FRQ2(int channel)
  *  Functions to wait for DMA streams to be ready.
  */
 /*
-static INLINE void wait_LR()
+static INLINE void wait_LR(void)
 {
    while (read_SMALR() & 0x80000000) ;
 }
@@ -844,7 +846,7 @@ static INLINE void wait_LR()
 
 
 /*
-static INLINE void wait_LW()
+static INLINE void wait_LW(void)
 {
    while (read_SMALW() & 0x80000000) ;
 }
@@ -853,7 +855,7 @@ static INLINE void wait_LW()
 
 
 /*
-static INLINE void wait_RR()
+static INLINE void wait_RR(void)
 {
    while (read_SMARR() & 0x80000000) ;
 }
@@ -862,7 +864,7 @@ static INLINE void wait_RR()
 
 
 /*
-static INLINE void wait_RW()
+static INLINE void wait_RW(void)
 {
    while (read_SMARW() & 0x80000000) ;
 }
@@ -895,7 +897,7 @@ static void write_init_arrays(unsigned short *init1, unsigned short *init2, unsi
 /* emu8k_init:
  *  Initialise the synthesiser. See AEPG chapter 4.
  */
-void emu8k_init()
+void emu8k_init(void)
 {
    int channel;
 
@@ -1082,12 +1084,12 @@ END_OF_STATIC_FUNCTION(emu8k_terminatesound);
  *  of the BLASTER environment variable, and then does some test reads to check
  *  that there is an EMU8000 there.
  */
-int emu8k_detect()
+int emu8k_detect(void)
 {
    char *envvar;
 
    if (!(envvar = getenv("BLASTER"))) {
-      ustrcpy(allegro_error, get_config_text("BLASTER environment variable not set"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("BLASTER environment variable not set"));
       return 0;
    }
 
@@ -1105,11 +1107,11 @@ int emu8k_detect()
    }
 
    if (!_emu8k_baseport) {
-      ustrcpy(allegro_error, get_config_text("BLASTER environment variable has no E section"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("BLASTER environment variable has no E section"));
       return 0;
    }
 
-   usprintf(allegro_error, get_config_text("AWE32 detection failed on port 0x%04x"), _emu8k_baseport);
+   uszprintf(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("AWE32 detection failed on port 0x%04x"), _emu8k_baseport);
 
    if ((read_word(7, 0, 3) & 0x000f) != 0x000c)
       return 0;
@@ -1372,7 +1374,7 @@ void emu8k_destroyenvelope(envparms_t * env)
 /* emu8k_lock:
  *  Locks all data and functions which might be called in an interrupt context.
  */
-void emu8k_lock()
+void emu8k_lock(void)
 {
    LOCK_VARIABLE(_emu8k_baseport);
    LOCK_VARIABLE(_emu8k_numchannels);
