@@ -26,8 +26,8 @@
 #include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
-#include "allegro/aintdos.h"
+#include "allegro/internal/aintern.h"
+#include "allegro/platform/aintdos.h"
 #include "grip.h"
 
 #ifndef ALLEGRO_DOS
@@ -126,7 +126,7 @@ static int read_grip(int j)
    for (s=0; s<joy[j].num_sticks; s++) {
       int reverse_factor;     /* messy way to account for up/down reversal */
 
-      if ((joy_type == JOY_TYPE_GRIP4) &&
+      if ((_joy_type == JOY_TYPE_GRIP4) &&
 	  (stick_class[j][s] == GRIP_CLASS_AXIS) &&
 	  (joy[j].stick[s].num_axis == 2)) {
 
@@ -162,7 +162,7 @@ static int read_grip(int j)
    }
 
    /* handle 4-way restriction */
-   if (joy_type == JOY_TYPE_GRIP4) {
+   if (_joy_type == JOY_TYPE_GRIP4) {
       for (s=0; s<joy[j].num_sticks; s++) {
 	 if ((stick_class[j][s] != GRIP_CLASS_AXIS) || 
 	     (joy[j].stick[s].num_axis != 2))
@@ -209,7 +209,7 @@ static int read_grip(int j)
 /* grip_init:
  *  Initializes the driver.
  */
-static int grip_init()
+static int grip_init(void)
 {
    static char *name_b[] =
    {
@@ -223,12 +223,12 @@ static int grip_init()
    int i = 0;
 
    if (grip_link()) {
-      usprintf(allegro_error, get_config_text("Could not find GRIP.GLL"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Could not find GRIP.GLL"));
       return -1;
    }
 
    if (!_GrInitialize()) {
-      usprintf(allegro_error, get_config_text("Could not initialize GrIP system"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Could not initialize GrIP system"));
       grip_unlink();
       return -1;
    }
@@ -249,7 +249,7 @@ static int grip_init()
    }
 
    if (num_joysticks == 0) {
-      usprintf(allegro_error, get_config_text("No GrIP devices available"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("No GrIP devices available"));
       grip_exit();
       return -1;
    }
@@ -288,7 +288,7 @@ static int grip_init()
 	    stick_class[i][stick] = GRIP_CLASS_AXIS;
 
 	    /* name stick... :( */
-	    strcpy(tmpstr, "Stick");
+	    _al_sane_strncpy(tmpstr, "Stick", sizeof(tmpstr));
 	    if (joy[i].num_sticks > 1) {
 	       tmpstr[strlen(tmpstr)+2] = '\0';
 	       tmpstr[strlen(tmpstr)+1] = '1'+stick;
@@ -302,7 +302,7 @@ static int grip_init()
 	    joy[i].stick[stick].num_axis = ((stick*2) == last_axis) ? 1 : 2;
 	    _GrGetControlName(slot_index[i], GRIP_CLASS_AXIS, stick*2, tmpstr);
 	    if (tmpstr[0] == '\0')
-	       strcpy(tmpstr, "X");
+	       _al_sane_strncpy(tmpstr, "X", sizeof(tmpstr));
 
 	    SIMPLIFY_AXIS(tmpstr);
 
@@ -314,7 +314,7 @@ static int grip_init()
 	    if (joy[i].stick[stick].num_axis == 2) {
 	       _GrGetControlName(slot_index[i], GRIP_CLASS_AXIS, stick*2 + 1, tmpstr);
 	       if (tmpstr[0] == '\0')
-		  strcpy(tmpstr, "Y");
+		  _al_sane_strncpy(tmpstr, "Y", sizeof(tmpstr));
 
 	       SIMPLIFY_AXIS(tmpstr);
 
@@ -348,7 +348,7 @@ static int grip_init()
 	    stick_class[i][stick] = GRIP_CLASS_ANALOG;
 
 	    /* name pad... :( */
-	    strcpy(tmpstr, "Analog");
+	    _al_sane_strncpy(tmpstr, "Analog", sizeof(tmpstr));
 	    if (num_pads > 1) {
 	       tmpstr[strlen(tmpstr)+2] = '\0';
 	       tmpstr[strlen(tmpstr)+1] = '1'+stick-analog_start;
@@ -362,7 +362,7 @@ static int grip_init()
 	    joy[i].stick[stick].num_axis = 2;
 	    _GrGetControlName(slot_index[i], GRIP_CLASS_ANALOG, (stick-analog_start)*2, tmpstr);
 	    if (tmpstr[0] == '\0')
-	       strcpy(tmpstr, "X");
+	       _al_sane_strncpy(tmpstr, "X", sizeof(tmpstr));
 
 	    SIMPLIFY_AXIS(tmpstr);
 
@@ -373,7 +373,7 @@ static int grip_init()
 
 	    _GrGetControlName(slot_index[i], GRIP_CLASS_ANALOG, (stick-analog_start)*2 + 1, tmpstr);
 	    if (tmpstr[0] == '\0')
-	       strcpy(tmpstr, "Y");
+	       _al_sane_strncpy(tmpstr, "Y", sizeof(tmpstr));
 
 	    SIMPLIFY_AXIS(tmpstr);
 
@@ -406,7 +406,7 @@ static int grip_init()
 	    stick_class[i][stick] = GRIP_CLASS_POV_HAT;
 
 	    /* name hat... :( */
-	    strcpy(tmpstr, "Hat");
+	    _al_sane_strncpy(tmpstr, "Hat", sizeof(tmpstr));
 	    if (num_hats > 1) {
 	       tmpstr[strlen(tmpstr)+2] = '\0';
 	       tmpstr[strlen(tmpstr)+1] = '1'+stick-pov_start;
@@ -420,7 +420,7 @@ static int grip_init()
 	    joy[i].stick[stick].num_axis = 2;
 	    _GrGetControlName(slot_index[i], GRIP_CLASS_POV_HAT, (stick-pov_start)*2, tmpstr);
 	    if (tmpstr[0] == '\0')
-	       strcpy(tmpstr, "X");
+	       _al_sane_strncpy(tmpstr, "X", sizeof(tmpstr));
 
 	    SIMPLIFY_AXIS(tmpstr);
 
@@ -431,7 +431,7 @@ static int grip_init()
 
 	    _GrGetControlName(slot_index[i], GRIP_CLASS_POV_HAT, (stick-pov_start)*2 + 1, tmpstr);
 	    if (tmpstr[0] == '\0')
-	       strcpy(tmpstr, "Y");
+	       _al_sane_strncpy(tmpstr, "Y", sizeof(tmpstr));
 
 	    SIMPLIFY_AXIS(tmpstr);
 
@@ -499,7 +499,7 @@ static int grip_init()
 	    joy[i].stick[stick].num_axis = last_axis;
 
 	    /* name throttle... */
-	    strcpy(tmpstr, "Throttle");
+	    _al_sane_strncpy(tmpstr, "Throttle", sizeof(tmpstr));
 	    if (num_throt_sticks > 1) {
 	       tmpstr[strlen(tmpstr)+2] = '\0';
 	       tmpstr[strlen(tmpstr)+1] = '1'+stick-throttle_start;
@@ -512,7 +512,7 @@ static int grip_init()
 	       /* name axis */
 	       _GrGetControlName(slot_index[i], GRIP_CLASS_THROTTLE, throttle_count, tmpstr);
 	       if (tmpstr[0] == '\0')
-		  strcpy(tmpstr, "Throttle");
+		  _al_sane_strncpy(tmpstr, "Throttle", sizeof(tmpstr));
 
 	       joy[i].stick[stick].axis[a].name = get_config_text(tmpstr);
 	       joy[i].stick[stick].axis[a].pos = 0;
@@ -540,7 +540,7 @@ static int grip_init()
 	 for (b=0; b<joy[i].num_buttons; b++) {
 	    _GrGetControlName(slot_index[i], GRIP_CLASS_BUTTON, b, tmpstr);
 	    if (tmpstr[0] == '\0')
-	       strcpy(tmpstr, name_b[b]);
+	       _al_sane_strncpy(tmpstr, name_b[b], sizeof(tmpstr));
 	    joy[i].button[b].name = get_config_text(tmpstr);
 	    joy[i].button[b].b = 0;
 	 }
@@ -557,7 +557,7 @@ static int grip_init()
 /* grip_exit:
  *  Shuts down the driver.
  */
-static void grip_exit()
+static void grip_exit(void)
 {
    _GrShutdown();
    grip_unlink();
@@ -569,7 +569,7 @@ static void grip_exit()
  *  Updates the joystick status variables.
  *  Assumes GrIP driver has been validly initialized.
  */
-static int grip_poll()
+static int grip_poll(void)
 {
    int i, all_good;
 
@@ -593,9 +593,9 @@ static int already_linked = 0;
 /* The following intiallization routine is mostly taken from
  * the Gravis SDK.
  */
-static int grip_link()
+static int grip_link(void)
 {
-   char name[256], *s;
+   char name[1024], tmp[64], *s;
    PACKFILE *f;
    long size;
    void *image;
@@ -627,9 +627,9 @@ static int grip_link()
 
       s = getenv("GRIP");
       if (s) {
-	 do_uconvert(s, U_ASCII, name, U_CURRENT, sizeof(name));
+	 do_uconvert(s, U_ASCII, name, U_CURRENT, sizeof(name) - ucwidth(OTHER_PATH_SEPARATOR));
 	 put_backslash(name);
-	 ustrcat(name, uconvert_ascii("grip.gll", NULL));
+	 ustrzcat(name, sizeof(name), uconvert_ascii("grip.gll", tmp));
       }
    }
 
@@ -667,7 +667,7 @@ static int grip_link()
 
 
 
-static int grip_unlink()
+static int grip_unlink(void)
 {
    already_linked = 0;
 
