@@ -467,10 +467,10 @@ static int utf8_isok(int c)
 
 
 /* string format table, to allow user expansion with other encodings */
-UTYPE_INFO utypes[] =
+static UTYPE_INFO utypes[] =
 {
    { U_ASCII,    ascii_getc,    ascii_getx,    ascii_setc,    ascii_width,   ascii_cwidth,   ascii_isok,    1    },
-   { U_UTF8,     utf8_getc,     utf8_getx,     utf8_setc,     utf8_width,    utf8_cwidth,    utf8_isok,     6     },
+   { U_UTF8,     utf8_getc,     utf8_getx,     utf8_setc,     utf8_width,    utf8_cwidth,    utf8_isok,     4     },
    { U_UNICODE,  unicode_getc,  unicode_getx,  unicode_setc,  unicode_width, unicode_cwidth, unicode_isok,  2  },
    { U_ASCII_CP, ascii_cp_getc, ascii_cp_getx, ascii_cp_setc, ascii_width,   ascii_cwidth,   ascii_cp_isok, 1 },
    { 0,          NULL,          NULL,          NULL,          NULL,          NULL,           NULL,          0                     },
@@ -544,7 +544,7 @@ void set_uformat(int type)
 /* get_uformat:
  *  Returns the current text encoding format.
  */
-int get_uformat()
+int get_uformat(void)
 {
    return utype;
 }
@@ -696,7 +696,7 @@ void do_uconvert(AL_CONST char *s, int type, char *buf, int newtype, int size)
  */
 char *uconvert(AL_CONST char *s, int type, char *buf, int newtype, int size)
 {
-   static char static_buf[512];
+   static char static_buf[1024];
    ASSERT(s);
    ASSERT(size >= 0);
 
@@ -2913,7 +2913,7 @@ static int decode_format_string(char *buf, STRING_ARG *string_arg, AL_CONST char
 		  /* store current string position */
 		  pstr_pos = va_arg(args, int *);
 		  *pstr_pos = len;
-		  slen = 0;
+		  slen = -1;
 		  NEXT_C();
 		  break;
 
