@@ -22,6 +22,10 @@
 
 #include "wddraw.h"
 
+#define PREFIX_I                "al-wddbmp INFO: "
+#define PREFIX_W                "al-wddbmp WARNING: "
+#define PREFIX_E                "al-wddbmp ERROR: "
+
 
 /* The video bitmap allocation scheme works as follows:
  *  - the screen is allocated as a single DirectDraw surface (primary or overlay,
@@ -80,75 +84,75 @@ static char *dd_err(long err)
    switch (err) {
 
       case DD_OK:
-         strcpy(err_str, "DD_OK");
+         _al_sane_strncpy(err_str, "DD_OK", sizeof(err_str));
          break;
 
       case DDERR_GENERIC:
-         strcpy(err_str, "DDERR_GENERIC");
+         _al_sane_strncpy(err_str, "DDERR_GENERIC", sizeof(err_str));
          break;
 
      case DDERR_INCOMPATIBLEPRIMARY:
-         strcpy(err_str, "DDERR_INCOMPATIBLEPRIMARY");
+         _al_sane_strncpy(err_str, "DDERR_INCOMPATIBLEPRIMARY", sizeof(err_str));
          break;
 
      case DDERR_INVALIDCAPS:
-         strcpy(err_str, "DDERR_INVALIDCAPS");
+         _al_sane_strncpy(err_str, "DDERR_INVALIDCAPS", sizeof(err_str));
          break;
 
       case DDERR_INVALIDOBJECT:
-         strcpy(err_str, "DDERR_INVALIDOBJECT");
+         _al_sane_strncpy(err_str, "DDERR_INVALIDOBJECT", sizeof(err_str));
          break;
 
       case DDERR_INVALIDPARAMS:
-         strcpy(err_str, "DDERR_INVALIDPARAMS");
+         _al_sane_strncpy(err_str, "DDERR_INVALIDPARAMS", sizeof(err_str));
          break;
 
      case DDERR_INVALIDPIXELFORMAT:
-         strcpy(err_str, "DDERR_INVALIDPIXELFORMAT");
+         _al_sane_strncpy(err_str, "DDERR_INVALIDPIXELFORMAT", sizeof(err_str));
          break;
 
       case DDERR_NOFLIPHW:
-         strcpy(err_str, "DDERR_NOFLIPHW");
+         _al_sane_strncpy(err_str, "DDERR_NOFLIPHW", sizeof(err_str));
          break;
 
       case DDERR_NOTFLIPPABLE:
-         strcpy(err_str, "DDERR_NOTFLIPPABLE");
+         _al_sane_strncpy(err_str, "DDERR_NOTFLIPPABLE", sizeof(err_str));
          break;
 
       case DDERR_OUTOFMEMORY:
-         strcpy(err_str, "DDERR_OUTOFMEMORY");
+         _al_sane_strncpy(err_str, "DDERR_OUTOFMEMORY", sizeof(err_str));
          break;
 
       case DDERR_OUTOFVIDEOMEMORY:
-         strcpy(err_str, "DDERR_OUTOFVIDEOMEMORY");
+         _al_sane_strncpy(err_str, "DDERR_OUTOFVIDEOMEMORY", sizeof(err_str));
          break;
 
      case DDERR_PRIMARYSURFACEALREADYEXISTS:
-         strcpy(err_str, "DDERR_PRIMARYSURFACEALREADYEXISTS");
+         _al_sane_strncpy(err_str, "DDERR_PRIMARYSURFACEALREADYEXISTS", sizeof(err_str));
          break;
 
       case DDERR_SURFACEBUSY:
-         strcpy(err_str, "DDERR_SURFACEBUSY");
+         _al_sane_strncpy(err_str, "DDERR_SURFACEBUSY", sizeof(err_str));
          break;
 
       case DDERR_SURFACELOST:
-         strcpy(err_str, "DDERR_SURFACELOST");
+         _al_sane_strncpy(err_str, "DDERR_SURFACELOST", sizeof(err_str));
          break;
 
      case DDERR_UNSUPPORTED:
-         strcpy(err_str, "DDERR_UNSUPPORTED");
+         _al_sane_strncpy(err_str, "DDERR_UNSUPPORTED", sizeof(err_str));
          break;
 
      case DDERR_UNSUPPORTEDMODE:
-         strcpy(err_str, "DDERR_UNSUPPORTEDMODE");
+         _al_sane_strncpy(err_str, "DDERR_UNSUPPORTEDMODE", sizeof(err_str));
          break;
 
      case DDERR_WASSTILLDRAWING:
-         strcpy(err_str, "DDERR_WASSTILLDRAWING");
+         _al_sane_strncpy(err_str, "DDERR_WASSTILLDRAWING", sizeof(err_str));
          break;
 
       default:
-         strcpy(err_str, "DDERR_UNKNOWN");
+         _al_sane_strncpy(err_str, "DDERR_UNKNOWN", sizeof(err_str));
          break;
    }
 
@@ -168,7 +172,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
 {
    DDSURFACEDESC ddsurf_desc;
    LPDIRECTDRAWSURFACE ddsurf1;
-   LPDIRECTDRAWSURFACE2 ddsurf2;
+   LPVOID ddsurf2;
    HRESULT hr;
 
    /* describe surface characteristics */
@@ -180,7 +184,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
 
       case DDRAW_SURFACE_PRIMARY:
          ddsurf_desc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
-         _TRACE("Creating primary surface...");
+         _TRACE(PREFIX_I "Creating primary surface...");
          break;
 
       case DDRAW_SURFACE_OVERLAY:
@@ -194,7 +198,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
             ddsurf_desc.ddpfPixelFormat = *pixel_format;
          }
 
-         _TRACE("Creating overlay surface...");
+         _TRACE(PREFIX_I "Creating overlay surface...");
          break;
 
       case DDRAW_SURFACE_VIDEO:
@@ -202,7 +206,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
          ddsurf_desc.dwFlags |= DDSD_HEIGHT | DDSD_WIDTH;
          ddsurf_desc.dwHeight = h;
          ddsurf_desc.dwWidth = w;
-         _TRACE("Creating video surface...");
+         _TRACE(PREFIX_I "Creating video surface...");
          break;
 
       case DDRAW_SURFACE_SYSTEM:
@@ -216,11 +220,11 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
             ddsurf_desc.ddpfPixelFormat = *pixel_format;
          }
 
-         _TRACE("Creating system surface...");
+         _TRACE(PREFIX_I "Creating system surface...");
          break;
 
       default:
-         _TRACE("Unknown surface type\n");
+         _TRACE(PREFIX_E "Unknown surface type\n");
          return NULL;
    }
 
@@ -235,12 +239,12 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
    /* create the surface */
    hr = IDirectDraw2_CreateSurface(directdraw, &ddsurf_desc, &ddsurf1, NULL);
    if (FAILED(hr)) {
-      _TRACE("Unable to create the surface (%s)\n", dd_err(hr));
+      _TRACE(PREFIX_E "Unable to create the surface (%s)\n", dd_err(hr));
       return NULL;
    }
 
    /* retrieve the DirectDrawSurface2 interface */
-   hr = IDirectDrawSurface_QueryInterface(ddsurf1, &IID_IDirectDrawSurface2, (LPVOID *)&ddsurf2);
+   hr = IDirectDrawSurface_QueryInterface(ddsurf1, &IID_IDirectDrawSurface2, &ddsurf2);
 
    /* there is a bug in the COM part of DirectX 3:
     *  If we release the DirectSurface interface, the actual
@@ -250,11 +254,11 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
       IDirectDrawSurface_Release(ddsurf1);
 
    if (FAILED(hr)) {
-      _TRACE("Unable to retrieve the DirectDrawSurface2 interface (%s)\n", dd_err(hr));
+      _TRACE(PREFIX_E "Unable to retrieve the DirectDrawSurface2 interface (%s)\n", dd_err(hr));
       return NULL;
    }
 
-   return ddsurf2;
+   return (LPDIRECTDRAWSURFACE2)ddsurf2;
 }
 
 
@@ -299,10 +303,10 @@ void gfx_directx_destroy_surface(DDRAW_SURFACE *surf)
 
 
 
-/* make_bitmap_from_surface:
+/* gfx_directx_make_bitmap_from_surface:
  *  Connects a DirectDraw surface with an Allegro bitmap.
  */
-BITMAP *make_bitmap_from_surface(DDRAW_SURFACE *surf, int w, int h, int id)
+BITMAP *gfx_directx_make_bitmap_from_surface(DDRAW_SURFACE *surf, int w, int h, int id)
 {
    BITMAP *bmp;
    int i;
@@ -330,7 +334,7 @@ BITMAP *make_bitmap_from_surface(DDRAW_SURFACE *surf, int w, int h, int id)
    for (i = 0; i < h; i++)
       bmp->line[i] = pseudo_surf_mem;
 
-   DDRAW_SURFACE_OF(bmp) = surf;
+   bmp->extra = surf;
 
    return bmp;
 }
@@ -358,8 +362,8 @@ static int recreate_flipping_chain(int n_pages)
    ASSERT(n_pages > 0);
 
    /* set flipping chain characteristics */
-   w = forefront_bitmap->w;
-   h = forefront_bitmap->h;
+   w = gfx_directx_forefront_bitmap->w;
+   h = gfx_directx_forefront_bitmap->h;
    type = flipping_page[0]->flags & DDRAW_SURFACE_TYPE_MASK;
    n_backbuffers = n_pages - 1;
 
@@ -367,7 +371,7 @@ static int recreate_flipping_chain(int n_pages)
    if (flipping_page[0]->id) {
       hr = IDirectDrawSurface2_Release(flipping_page[0]->id);
       if (FAILED(hr)) {
-         _TRACE("Unable to release the primary surface (%s)", dd_err(hr));
+         _TRACE(PREFIX_E "Unable to release the primary surface (%s)", dd_err(hr));
          return -1;
       }
    }
@@ -385,7 +389,7 @@ static int recreate_flipping_chain(int n_pages)
       ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
       hr = IDirectDrawSurface2_GetAttachedSurface(flipping_page[0]->id, &ddscaps, &flipping_page[1]->id);
       if (FAILED(hr)) {
-         _TRACE("Unable to retrieve the first backbuffer (%s)", dd_err(hr));
+         _TRACE(PREFIX_E "Unable to retrieve the first backbuffer (%s)", dd_err(hr));
          return -1;
       }
 
@@ -397,7 +401,7 @@ static int recreate_flipping_chain(int n_pages)
          ddscaps.dwCaps = DDSCAPS_FLIP;
          hr = IDirectDrawSurface2_GetAttachedSurface(flipping_page[1]->id, &ddscaps, &flipping_page[2]->id);
          if (FAILED(hr)) {
-            _TRACE("Unable to retrieve the second backbuffer (%s)", dd_err(hr));
+            _TRACE(PREFIX_E "Unable to retrieve the second backbuffer (%s)", dd_err(hr));
             return -1;
          }
 
@@ -410,7 +414,7 @@ static int recreate_flipping_chain(int n_pages)
    if (flipping_page[0]->flags & DDRAW_SURFACE_INDEXED) {
       hr = IDirectDrawSurface2_SetPalette(flipping_page[0]->id, ddpalette);
       if (FAILED(hr)) {
-         _TRACE("Unable to attach the global palette (%s)", dd_err(hr));
+         _TRACE(PREFIX_E "Unable to attach the global palette (%s)", dd_err(hr));
          return -1;
       }
    }
@@ -428,14 +432,14 @@ BITMAP *gfx_directx_create_video_bitmap(int width, int height)
    BITMAP *bmp;
 
    /* try to detect page flipping and triple buffering patterns */
-   if ((width == forefront_bitmap->w) && (height == forefront_bitmap->h)) {
+   if ((width == gfx_directx_forefront_bitmap->w) && (height == gfx_directx_forefront_bitmap->h)) {
 
       switch (n_flipping_pages) {
 
          case 0:
             /* recycle the forefront surface as the first flipping page */
-            flipping_page[0] = DDRAW_SURFACE_OF(forefront_bitmap);
-            bmp = make_bitmap_from_surface(flipping_page[0], width, height, BMP_ID_VIDEO);
+            flipping_page[0] = DDRAW_SURFACE_OF(gfx_directx_forefront_bitmap);
+            bmp = gfx_directx_make_bitmap_from_surface(flipping_page[0], width, height, BMP_ID_VIDEO);
             if (bmp) {
                flipping_page[0]->parent_bmp = bmp;
                n_flipping_pages++;
@@ -451,7 +455,7 @@ BITMAP *gfx_directx_create_video_bitmap(int width, int height)
             /* try to attach an additional page to the flipping chain */
             flipping_page[n_flipping_pages] = malloc(sizeof(DDRAW_SURFACE));
             if (recreate_flipping_chain(n_flipping_pages+1) == 0) {
-               bmp = make_bitmap_from_surface(flipping_page[n_flipping_pages], width, height, BMP_ID_VIDEO);
+               bmp = gfx_directx_make_bitmap_from_surface(flipping_page[n_flipping_pages], width, height, BMP_ID_VIDEO);
                if (bmp) {
                   flipping_page[n_flipping_pages]->parent_bmp = bmp;
                   n_flipping_pages++;
@@ -476,7 +480,7 @@ BITMAP *gfx_directx_create_video_bitmap(int width, int height)
       return NULL;
 
    /* create the bitmap that wraps up the surface */
-   bmp = make_bitmap_from_surface(surf, width, height, BMP_ID_VIDEO);
+   bmp = gfx_directx_make_bitmap_from_surface(surf, width, height, BMP_ID_VIDEO);
    if (!bmp) {
       gfx_directx_destroy_surface(surf);
       return NULL;
@@ -506,7 +510,7 @@ void gfx_directx_destroy_video_bitmap(BITMAP *bmp)
           */
          if (surf != tail_page) {
             surf->parent_bmp = tail_page->parent_bmp;
-            DDRAW_SURFACE_OF(surf->parent_bmp) = surf;
+            surf->parent_bmp->extra = surf;
          }
 
          /* remove the tail page from the flipping chain */
@@ -534,8 +538,8 @@ static int flip_with_forefront_bitmap(BITMAP *bmp, int wait)
    HRESULT hr;
 
    /* flip only in the foreground */
-   if (!app_foreground) {
-      thread_switch_out();
+   if (!_win_app_foreground) {
+      _win_thread_switch_out();
       return 0;
    }
 
@@ -556,17 +560,17 @@ static int flip_with_forefront_bitmap(BITMAP *bmp, int wait)
    }
 
    if (FAILED(hr)) {
-      _TRACE("Can't flip (%s)\n", dd_err(hr));
+      _TRACE(PREFIX_E "Can't flip (%s)\n", dd_err(hr));
       return -1;
    }
 
    /* attach the surface to the former forefront bitmap */
    surf->parent_bmp = flipping_page[0]->parent_bmp;
-   DDRAW_SURFACE_OF(surf->parent_bmp) = surf;
+   surf->parent_bmp->extra = surf;
 
    /* make the bitmap point to the forefront surface */
    flipping_page[0]->parent_bmp = bmp;
-   DDRAW_SURFACE_OF(bmp) = flipping_page[0];
+   bmp->extra = flipping_page[0];
 
    return 0;
 }
@@ -578,7 +582,7 @@ static int flip_with_forefront_bitmap(BITMAP *bmp, int wait)
 int gfx_directx_show_video_bitmap(BITMAP *bmp)
 {
    /* guard against show_video_bitmap(screen); */
-   if (bmp == forefront_bitmap)
+   if (bmp == gfx_directx_forefront_bitmap)
       return 0;
 
    return flip_with_forefront_bitmap(bmp, _wait_for_vsync);
@@ -591,7 +595,7 @@ int gfx_directx_show_video_bitmap(BITMAP *bmp)
 int gfx_directx_request_video_bitmap(BITMAP *bmp)
 {
    /* guard against request_video_bitmap(screen); */
-   if (bmp == forefront_bitmap)
+   if (bmp == gfx_directx_forefront_bitmap)
       return 0;
 
    return flip_with_forefront_bitmap(bmp, FALSE);
@@ -636,7 +640,7 @@ BITMAP *gfx_directx_create_system_bitmap(int width, int height)
       return NULL;
 
    /* create the bitmap that wraps up the surface */
-   bmp = make_bitmap_from_surface(surf, width, height, BMP_ID_SYSTEM);
+   bmp = gfx_directx_make_bitmap_from_surface(surf, width, height, BMP_ID_SYSTEM);
    if (!bmp) {
       gfx_directx_destroy_surface(surf);
       return NULL;
