@@ -53,6 +53,7 @@ GFX_DRIVER gfx_directx_accel =
    NULL,                         // AL_METHOD(void, drawing_mode, (void));
    NULL,                         // AL_METHOD(void, save_video_state, (void*));
    NULL,                         // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                         // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    gfx_directx_fetch_mode_list,
    0, 0,                         // physical (not virtual!) screen size
    TRUE,                         // true if video memory is linear
@@ -91,6 +92,7 @@ GFX_DRIVER gfx_directx_soft =
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    gfx_directx_fetch_mode_list,
    0, 0,                        // physical (not virtual!) screen size
    TRUE,                        // true if video memory is linear
@@ -124,6 +126,7 @@ GFX_DRIVER gfx_directx_safe =
    NULL,                        // AL_METHOD(void, drawing_mode, (void));
    NULL,                        // AL_METHOD(void, save_video_state, (void*));
    NULL,                        // AL_METHOD(void, restore_video_state, (void*));
+   NULL,                        // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    gfx_directx_fetch_mode_list,
    0, 0,                        // physical (not virtual!) screen size
    TRUE,                        // true if video memory is linear
@@ -161,8 +164,8 @@ static struct BITMAP *init_directx_accel(int w, int h, int v_w, int v_h, int col
    bmp = gfx_directx_init(&gfx_directx_accel, w, h, v_w, v_h, color_depth);
 
    if (bmp) {
-      enable_acceleration(&gfx_directx_accel);
-      enable_triple_buffering(&gfx_directx_accel);
+      gfx_directx_enable_acceleration(&gfx_directx_accel);
+      gfx_directx_enable_triple_buffering(&gfx_directx_accel);
       finalize_fullscreen_init();
    }
 
@@ -185,7 +188,7 @@ static struct BITMAP *init_directx_soft(int w, int h, int v_w, int v_h, int colo
    bmp = gfx_directx_init(&gfx_directx_soft, w, h, v_w, v_h, color_depth);
 
    if (bmp) {
-      enable_triple_buffering(&gfx_directx_soft);
+      gfx_directx_enable_triple_buffering(&gfx_directx_soft);
       finalize_fullscreen_init();
    }
 
