@@ -24,6 +24,8 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <unistd.h>
 
 
@@ -37,6 +39,7 @@ static int _xwin_sysdrv_init(void);
 static void _xwin_sysdrv_exit(void);
 static void _xwin_sysdrv_set_window_title(AL_CONST char *name);
 static int _xwin_sysdrv_set_close_button_callback(void (*proc)(void));
+static void _xwin_sysdrv_message(AL_CONST char *msg);
 static int _xwin_sysdrv_display_switch_mode(int mode);
 static int _xwin_sysdrv_desktop_color_depth(void);
 static int _xwin_sysdrv_get_desktop_resolution(int *width, int *height);
@@ -65,7 +68,7 @@ SYSTEM_DRIVER system_xwin =
    _unix_find_resource,
    _xwin_sysdrv_set_window_title,
    _xwin_sysdrv_set_close_button_callback,
-   NULL, /* message */
+   _xwin_sysdrv_message,
    NULL, /* assert */
    NULL, /* save_console_state */
    NULL, /* restore_console_state */
@@ -162,7 +165,7 @@ static int _xwin_sysdrv_init(void)
 {
    char tmp[256];
 
-   _read_os_type();
+   _unix_read_os_type();
 
    /* install emergency-exit signal handlers */
    old_sig_abrt = signal(SIGABRT, _xwin_signal_handler);
@@ -194,10 +197,12 @@ static int _xwin_sysdrv_init(void)
       return -1;
    }
 
-   /* If multithreaded bg_man, need to init X's lock/unlock facility. 
-    * Note that no X calls must be made before this point! */
+#ifdef ALLEGRO_MULTITHREADED
    if (_unix_bg_man->multi_threaded)
-      XInitThreads();
+   {
+       _xwin.mutex = _unix_create_mutex ();
+   }
+#endif
 
    get_executable_name(tmp, sizeof(tmp));
    set_window_title(get_filename(tmp));
@@ -222,6 +227,12 @@ static int _xwin_sysdrv_init(void)
  */
 static void _xwin_sysdrv_exit(void)
 {
+   /* This stops the X event handler running in the background, which
+    * seems a nice thing to do before closing the connection to the X
+    * display... (remove this and you get SIGABRTs during XCloseDisplay).
+    */
+   _unix_bg_man->unregister_func(_xwin_bg_handler);
+
    _xwin_close_display();
    _unix_bg_man->exit();
 
@@ -238,6 +249,13 @@ static void _xwin_sysdrv_exit(void)
 #ifdef SIGQUIT
    signal(SIGQUIT, old_sig_quit);
 #endif
+
+#ifdef ALLEGRO_MULTITHREADED
+   if (_xwin.mutex) {
+      _unix_destroy_mutex(_xwin.mutex);
+      _xwin.mutex = NULL;
+   }
+#endif
 }
 
 
@@ -268,6 +286,58 @@ static int _xwin_sysdrv_set_close_button_callback(void (*proc)(void))
 
 
 
+/* _xwin_sysdrv_message:
+ *  Displays a message. Uses xmessage if possible, and stdout if not.
+ */
+static void _xwin_sysdrv_message(AL_CONST char *msg)
+{
+   char buf[ALLEGRO_MESSAGE_SIZE+1];
+   char *msg2;
+   size_t len;
+   pid_t pid;
+   int status;
+
+   /* convert message to ASCII */
+   msg2 = uconvert(msg, U_CURRENT, buf, U_ASCII, ALLEGRO_MESSAGE_SIZE);
+
+   /* xmessage interprets some strings beginning with '-' as command-line
+    * options. To avoid this we make sure all strings we pass to it have
+    * newlines on the end. This is also useful for the fputs() case.
+    */
+   len = strlen(msg2);
+   ASSERT(len < ALLEGRO_MESSAGE_SIZE);
+   if ((len == 0) || (msg2[len-1] != '\n'))
+      strcat(msg2, "\n");
+
+   /* fork a child */
+   pid = fork();
+   switch (pid) {
+
+      case -1: /* fork error */
+	 fputs(msg2, stdout);
+	 break;
+
+      case 0: /* child process */
+	 execlp("xmessage", "xmessage", "-buttons", "OK:101", "-default", "OK", "-center", msg2, (char *)NULL);
+
+	 /* if execution reaches here, it means execlp failed */
+	 _exit(EXIT_FAILURE);
+	 break;
+
+      default: /* parent process */
+	 waitpid(pid, &status, 0);
+	 if ((!WIFEXITED(status))
+	     || (WEXITSTATUS(status) != 101)) /* ok button */
+	 {
+	    fputs(msg2, stdout);
+	 }
+
+	 break;
+   }
+}
+
+
+
 /* _xwin_sysdrv_gfx_drivers:
  *  Get the list of graphics drivers.
  */
@@ -347,17 +417,13 @@ static _DRIVER_INFO *_xwin_sysdrv_timer_drivers(void)
  */
 static int _xwin_sysdrv_display_switch_mode(int mode)
 {
-#ifdef ALLEGRO_XWINDOWS_WITH_XF86DGA
    if (_xwin.in_dga_mode) {
       if (mode != SWITCH_NONE)
 	 return -1;
    }
-   else 
-#endif
-   {
-      if (mode != SWITCH_BACKGROUND)
-	 return -1;
-   }
+
+   if (mode != SWITCH_BACKGROUND)
+      return -1;
 
    return 0;
 }
@@ -386,9 +452,10 @@ static int _xwin_sysdrv_desktop_color_depth(void)
  */
 static int _xwin_sysdrv_get_desktop_resolution(int *width, int *height)
 {
+   XLOCK();
    *width = DisplayWidth(_xwin.display, _xwin.screen);
    *height = DisplayHeight(_xwin.display, _xwin.screen);
-
+   XUNLOCK();
    return 0;
 }
 
