@@ -12,6 +12,8 @@
  *
  *      By Shawn Hargreaves.
  *
+ *      Nathan Smith added the recursive handling of directories.
+ *
  *      See readme.txt for copyright information.
  */
 
@@ -32,8 +34,6 @@
 
 
 
-static DATAFILE *datafile = NULL;
-
 static int err = 0;
 static int changed = 0;
 
@@ -58,20 +58,34 @@ static char *opt_prefixstring = NULL;
 static char *opt_password = NULL;
 static char *opt_palette = NULL;
 
+static int attrib_ok = FA_ARCH|FA_RDONLY;
+
 static char canonical_datafilename[1024];
 
-#define MAX_FILES    256
-
-static char *opt_proplist[MAX_FILES];
+static char **opt_proplist = NULL;
 static int opt_numprops = 0;
 
-static char *opt_fixedproplist[MAX_FILES];
+static char **opt_fixedproplist = NULL;
 static int opt_numfixedprops = 0;
 
-static char *opt_namelist[MAX_FILES];
+static char **opt_namelist = NULL;
 static int opt_numnames = 0;
 
-static int opt_usedname[MAX_FILES];
+static int *opt_usedname = NULL;
+
+
+
+static void allocate(int n)
+{
+   static int max = 0;
+   if (n <= max)
+      return;
+   opt_proplist = realloc(opt_proplist, n * sizeof *opt_proplist);
+   opt_fixedproplist = realloc(opt_fixedproplist, n * sizeof *opt_fixedproplist);
+   opt_namelist = realloc(opt_namelist, n * sizeof *opt_namelist);
+   opt_usedname = realloc(opt_usedname, n * sizeof *opt_usedname);
+   max = n;
+}
 
 
 
@@ -90,6 +104,7 @@ static void usage(void)
    printf("\t'-d' deletes the named objects from the datafile\n");
    printf("\t'-dither' dithers when reducing color depths\n");
    printf("\t'-e' extracts the named objects from the datafile\n");
+   printf("\t'-f' store references to original files as relative filenames\n");
    printf("\t'-g x y w h' grabs bitmap data from a specific grid location\n");
    printf("\t'-h outputfile.h' sets the output header file\n");
    printf("\t'-k' keeps the original filenames when grabbing objects\n");
@@ -100,7 +115,7 @@ static void usage(void)
    printf("\t'-o output' sets the output file or directory when extracting data\n");
    printf("\t'-p prefixstring' sets the prefix for the output header file\n");
    printf("\t'-pal objectname' specifies which palette to use\n");
-   printf("\t'-r' store references to original files as relative filenames\n");
+   printf("\t'-r' recursively adds directories as nested datafiles\n");
    printf("\t'-s0' no strip: save everything\n");
    printf("\t'-s1' strip grabber specific information from the file\n");
    printf("\t'-s2' strip all object properties and names from the file\n");
@@ -110,6 +125,7 @@ static void usage(void)
    printf("\t'-u' updates the contents of the datafile\n");
    printf("\t'-v' selects verbose mode\n");
    printf("\t'-w' always updates the entire contents of the datafile\n");
+   printf("\t'-x' alias for -e\n");
    printf("\t'-007 password' sets the file encryption key\n");
    printf("\t'PROP=value' sets object properties\n");
 }
@@ -256,6 +272,37 @@ int datedit_ask(AL_CONST char *fmt, ...)
 
 
 
+/* callback for the datedit functions to show a list of options */
+/* Returns -1 if canceled */
+int datedit_select(AL_CONST char *(*list_getter)(int index, int *list_size), AL_CONST char *fmt, ...)
+{
+   va_list args;
+   char buf[1024];
+   int c, num;
+
+   va_start(args, fmt);
+   vsprintf(buf, fmt, args);
+   va_end(args);
+
+   /* If there is only one choice, select it automatically. If the list is */
+   /*  empty, return immediately. */
+   list_getter(-1, &num);
+   if (num<=1) return num-1;
+   
+   printf("%s:\n", buf);
+   for (c=0; c<num; c++) {
+      printf("% 2d %s\n", c, list_getter(c, NULL));
+   }
+   printf("% 2d %s\n", -1, "Cancel");
+   fflush(stdout);
+   fscanf(stdin, "%d", &c);
+   if (c>num)
+      c = num-1;
+   return c;
+}
+
+
+
 /* checks if a string is one of the names specified on the command line */
 static int is_name(AL_CONST char *name)
 {
@@ -394,19 +441,68 @@ static void do_delete(DATAFILE **dat, char *parentname)
 
 
 
+/* open a sub-datafile in the parent datafile */
+static DATAFILE *open_sub_datafile(DATAFILE *parent, AL_CONST char *name)
+{
+   DATAFILE *dat;
+   int c;
+
+   /* Return the datafile if it already exists. */
+   for (c=0; parent[c].type != DAT_END; c++) {
+      if ((parent[c].type == DAT_FILE) && (stricmp(name, get_datafile_property(parent+c, DAT_NAME)) == 0))
+	 return parent[c].dat;
+   }
+
+   /* Otherwise create a new datafile. */
+   printf("Creating sub-datafile %s\n", name);
+
+   dat = malloc(sizeof(DATAFILE));
+   dat->dat = NULL;
+   dat->type = DAT_END;
+   dat->size = 0;
+   dat->prop = NULL;
+
+   return dat;
+}
+
+
+
+/* close a sub-datafile from the parent datafile */
+static DATAFILE *close_sub_datafile(DATAFILE *parent, AL_CONST char *name, DATAFILE *dat)
+{
+   int c;
+
+   /* Adjust the datafile if it already exists. */
+   for (c=0; parent[c].type != DAT_END; c++) {
+      if ((parent[c].type == DAT_FILE) && (stricmp(name, get_datafile_property(parent+c, DAT_NAME)) == 0)) {
+	 parent[c].dat = dat;
+	 return parent;
+      }
+   }
+
+   /* Otherwise insert the datafile. */
+   return datedit_insert(parent, NULL, name, DAT_FILE, dat, 0);
+}
+
+
+
 /* adds a file to the archive */
 static int do_add_file(AL_CONST char *filename, int attrib, void *param)
 {
+   DATEDIT_GRAB_PARAMETERS params;
+   DATAFILE **dat = param;
+   DATAFILE *d;
    char fname[256];
    char name[256];
    int c;
-   DATAFILE *d;
-   DATEDIT_GRAB_PARAMETERS params;
 
    canonicalize_filename(fname, filename, sizeof(fname));   
 
    strcpy(name, get_filename(fname));
 
+   if ((strcmp(name, ".") == 0) || (strcmp(name, "..") == 0))
+      return 0;
+
    if (!opt_keepnames) {
       strupr(name);
 
@@ -415,6 +511,28 @@ static int do_add_file(AL_CONST char *filename, int attrib, void *param)
 	    name[c] = '_';
    }
 
+   if (attrib & FA_DIREC) {
+      DATAFILE *sub_dat = open_sub_datafile(*dat, name);
+
+      if (!sub_dat) {
+	 fprintf(stderr, "Error: failed to create sub-datafile %s\n", name);
+	 err = 1;
+	 return -1;
+      }
+
+      append_filename(fname, fname, "*", sizeof(fname));
+
+      if (for_each_file_ex(fname, 0, ~attrib_ok, do_add_file, &sub_dat) <= 0) {
+	 fprintf(stderr, "Error: failed to add objects to %s\n", name);
+	 err = 1;
+	 return -1;
+      }
+
+      *dat = close_sub_datafile(*dat, name, sub_dat);
+      changed = TRUE;
+      return 0;
+   }
+
    params.datafile = canonical_datafilename;
    params.filename = fname;
    params.name = name;
@@ -426,10 +544,10 @@ static int do_add_file(AL_CONST char *filename, int attrib, void *param)
    params.colordepth = opt_colordepth;
    params.relative = opt_relf;
 
-   for (c=0; datafile[c].type != DAT_END; c++) {
-      if (stricmp(name, get_datafile_property(datafile+c, DAT_NAME)) == 0) {
+   for (c=0; (*dat)[c].type != DAT_END; c++) {
+      if (stricmp(name, get_datafile_property(*dat+c, DAT_NAME)) == 0) {
 	 printf("Replacing %s -> %s\n", fname, name);
-	 if (!datedit_grabreplace(datafile+c, &params)) {
+	 if (!datedit_grabreplace(*dat+c, &params)) {
 	    err = 1;
 	    return -1;
 	 }
@@ -441,13 +559,13 @@ static int do_add_file(AL_CONST char *filename, int attrib, void *param)
    }
 
    printf("Inserting %s -> %s\n", fname, name);
-   d = datedit_grabnew(datafile, &params);
+   d = datedit_grabnew(*dat, &params);
    if (!d) {
       err = 1;
       return -1;
    }
    else {
-      datafile = d;
+      *dat = d;
       changed = TRUE;
       return 0;
    }
@@ -680,6 +798,7 @@ int main(int argc, char *argv[])
 {
    int c, colorconv_mode = 0;
    int *opt_fixed_prop = NULL;
+   DATAFILE *datafile = NULL;
 
    if (install_allegro(SYSTEM_NONE, &errno, atexit) != 0)
       return 1;
@@ -736,6 +855,10 @@ int main(int argc, char *argv[])
 	       opt_compression = argv[c][2] - '0'; 
 	       break;
 
+	    case 'f':
+	       opt_relf = TRUE;
+	       break;
+
 	    case 'g':
 	       if ((opt_gridx > 0) || (c >= argc-4)) {
 		  usage();
@@ -807,15 +930,15 @@ int main(int argc, char *argv[])
 		  opt_prefixstring = argv[++c];
 	       }
 	       break;
-               
+
 	    case 'r':
-               opt_relf = TRUE;
-               break;
+	       attrib_ok |= FA_DIREC;
+	       break;
 
 	    case 's':
 	       if (argv[c][2] == '-') {
-		  if (opt_numfixedprops < MAX_FILES)
-		     opt_fixedproplist[opt_numfixedprops++] = argv[c]+3;
+		  allocate(opt_numfixedprops + 1);
+		  opt_fixedproplist[opt_numfixedprops++] = argv[c]+3;
 	       }
 	       else {
 		  if ((opt_strip >= 0) || 
@@ -870,18 +993,17 @@ int main(int argc, char *argv[])
       }
       else {
 	 if (strchr(argv[c], '=')) {
-	    if (opt_numprops < MAX_FILES)
-	       opt_proplist[opt_numprops++] = argv[c];
+	    allocate(opt_numprops + 1);
+	    opt_proplist[opt_numprops++] = argv[c];
 	 }
 	 else {
 	    if (!opt_datafilename)
 	       opt_datafilename = argv[c];
 	    else {
-	       if (opt_numnames < MAX_FILES) {
-		  opt_namelist[opt_numnames] = argv[c];
-		  opt_usedname[opt_numnames] = FALSE;
-		  opt_numnames++;
-	       }
+	       allocate(opt_numnames + 1);
+	       opt_namelist[opt_numnames] = argv[c];
+	       opt_usedname[opt_numnames] = FALSE;
+	       opt_numnames++;
 	    }
 	 }
       }
@@ -923,7 +1045,7 @@ int main(int argc, char *argv[])
 	       }
 	       else {
 		  for (c=0; c<opt_numnames; c++) {
-		     if (for_each_file_ex(opt_namelist[c], 0, ~(FA_ARCH | FA_RDONLY), do_add_file, NULL) <= 0) {
+		     if (for_each_file_ex(opt_namelist[c], 0, ~attrib_ok, do_add_file, &datafile) <= 0) {
 			fprintf(stderr, "Error: %s not found\n", opt_namelist[c]);
 			err = 1;
 			break;
@@ -1044,4 +1166,4 @@ int main(int argc, char *argv[])
    return err;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
